
#include "stdafx.h"
#include "sdlacfg.h"

////////////////////////////////////////////////////////////////////////////////
//AFT cfg functions prototypes

int 
S514_OnNotify(	
	HWND		ParentHwnd,
	LPNMHDR		NmHdr,
	SETUPSTUFF	*Params
	);

void 
S514_Init(
	HKEY	hkey, 
	HWND	hdlg,
	GLOBALS	*cfg
	);

static int selected_ind=0;
LRESULT CALLBACK logic_ch_properties_dlg(HWND hdlg, UINT iMessage,
										 WPARAM wParam, LPARAM lParam);

static BOOL s514_check_and_save_cfg(HWND hdlg);


///////////////////////////////////////////////////////////////////////////////

#define NUM_OF_OPER_MODE_OPTIONS	2

char* s514_operational_mode_options[NUM_OF_OPER_MODE_OPTIONS] = 
	{	SDLA_DECODE_USEDBY_FIELD(API),
		SDLA_DECODE_USEDBY_FIELD(STACK)	//select this, if another driver
										//(like sprotocol.sys or wanpipe.sys) will run above. 
	};

//these are recommendations only:
#define MIN_RECOMMENDED_BITSTREAM_MTU	256
#define MIN_RECOMMENDED_HDLC_MTU		2048

///////////////////////////////////////////////////////////////////////////////
BOOL WINAPI S514_PageDlgProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam)
{							// PageDlgProc
	switch (msg)
	{						// process message
	case WM_NOTIFY:
		Debug(DBG_S514_PAGE, "WM_NOTIFY");

		//If there is a problem with the new configuration 
		//refuse to close the property page and return TRUE.
		//If all checks are OK allow the closing.
        if(S514_OnNotify(hdlg, (NMHDR *)lParam, stuff) == HANDLE_ERROR)
		{
			Debug(DBG_S514_PAGE, "Invalid Configuration, refusing to close!");
			SetWindowLong(hdlg, DWL_MSGRESULT, PSNRET_INVALID);
			return TRUE;
		}
		break;

	case WM_INITDIALOG:
		{
			if (okay == TRUE)
			{
				//initialize visual controls
				S514_Init(hkey, hdlg, &Globals_New_Cfg);
			}

			if (!okay)
			{					// no SampleInfo
				ShowWindow(GetDlgItem(hdlg, IDC_INFOLABEL), SW_HIDE);
				ShowWindow(GetDlgItem(hdlg, IDB_MOREINFO), SW_HIDE);
			}					// no SampleInfo
		}
		break;
		
   	case WM_DESTROY:
		Debug(DBG_S514_PAGE, "Samclass:WM_DESTROY");
		{
			LPSTR infofile = (LPSTR) GetWindowLong(hdlg, DWL_USER);
			if (infofile){
				GlobalFree((HGLOBAL) infofile);
			}
		}
		break;

	case WM_SHOWWINDOW:
		//This message received each time the Dialog shown/hidden.
		//Update visual controls with latest information stored in "Globals_New_Cfg".
		Debug(DBG_S514_PAGE, "DBG_S514_PAGE:WM_SHOWWINDOW");
		set_dialog_wanpipe_name(hdlg);
		S514_Init(hkey, hdlg, &Globals_New_Cfg);
		break;

    case WM_COMMAND:
		switch (LOWORD(wParam))
		{					// process command notification
		case BUTTON_HELP:
			WinHelp(GetActiveWindow(), "WanpipePnp.hlp", HELP_CONTEXT,
				IDH_LINE_CFG);
			break;

		case BUTTON_START_MONITOR:
			start_monitor(selected_ind);
			break;
		}					// process command notification
		return TRUE;
	}						// process message

	return FALSE;
}							// PageDlgProc

///////////////////////////////////////////////////////////////////////////////

UINT CALLBACK S514_PageCallbackProc(HWND junk, UINT msg, LPPROPSHEETPAGE p)
{							// PageCallbackProc
	if (msg == PSPCB_RELEASE && p->lParam)
	{
		Debug(DBG_S514_PAGE, "PSPCB_RELEASE : S514_PageCallbackProc()");
	}
	return TRUE;				// matters only for PSPCB_CREATE
}							// PageCallbackProc

///////////////////////////////////////////////////////////////////////////////

int S514_OnNotify(	HWND    ParentHwnd,
					LPNMHDR NmHdr,
					SETUPSTUFF * Params	)
{
    SP_DEVINSTALL_PARAMS spDevInstall = {0};
    TCHAR                friendlyName[LINE_LEN] ={0};
	BOOL	configuration_check_passed = FALSE;
    
    switch (NmHdr->code) 
	{
    case PSN_APPLY:
        //
        // Sent when the user clicks on Apply OR OK !!
        //
		configuration_check_passed = s514_check_and_save_cfg(ParentHwnd);

		Debug(DBG_S514_PAGE, "AFT_OnNotify((): PSN_APPLY");

		released_by_OK_button = 1;

		if(configuration_check_passed == TRUE){
			return HANDLE_OK;
		}else{
			return HANDLE_ERROR;
		}
		break;

	case PSN_QUERYCANCEL:
		Debug(DBG_S514_PAGE, "AFT_OnNotify(() : PSN_QUERYCANCEL");
		released_by_OK_button = 0;
    default:
		Debug(DBG_S514_PAGE, "AFT_OnNotify(() : default case:");
        return HANDLE_DEFAULT;
    }
    return HANDLE_DEFAULT;   
}

///////////////////////////////////////////////////////////////////////////////

static BOOL s514_check_and_save_cfg(HWND hdlg)
{	
	unsigned int	mtu, j;
	char			local_tmp_buff[TMP_BUFFER_LEN];

	Debug(DBG_S514_PAGE, "s514_check_and_save_cfg()");
	
	Debug(DBG_S514_PAGE, "*********  Writing S514 Interface Configuration  ********\n");

	///////////////////////////////////////////////////////////////////
	//get MTU
	GetDlgItemText(hdlg, EDIT_MTU,
		Globals_New_Cfg.aft_logic_ch_cfg[selected_ind].mtu,
		TMP_BUFFER_LEN);

	mtu = atoi(Globals_New_Cfg.aft_logic_ch_cfg[selected_ind].mtu);
	if(mtu < MIN_AFT_MTU || mtu > MAX_AFT_MTU){

		_snprintf(local_tmp_buff, TMP_BUFFER_LEN,
			"Invalid value in \"MTU\". Minimum is %d, Maximum is %d.",
			MIN_AFT_MTU, MAX_AFT_MTU);

		MessageBox(GetActiveWindow(), local_tmp_buff, "Configuration Error",															//szTitleString,
			MB_OK|MB_ICONWARNING|MB_SYSTEMMODAL);
		return FALSE;
	}

	if(mtu % 4){
		_snprintf(local_tmp_buff, TMP_BUFFER_LEN,
			"Invalid value in \"MTU\". Must be divizible by 4.");

		MessageBox(GetActiveWindow(), local_tmp_buff, "Configuration Error",															//szTitleString,
			MB_OK|MB_ICONWARNING|MB_SYSTEMMODAL);
		return FALSE;
	}

	if(mtu < MIN_RECOMMENDED_HDLC_MTU){

		_snprintf(local_tmp_buff, TMP_BUFFER_LEN,
			"The recommended \"MTU\" for HDLC is %d. Current \"MTU\" is %d. Would you like to use the recommended \"MTU\"?",
			MIN_RECOMMENDED_HDLC_MTU, mtu);

		if(yes_no_question(local_tmp_buff) == IDYES){
			//set the MTU value
			_snprintf(	Globals_New_Cfg.aft_logic_ch_cfg[selected_ind].mtu, TMP_BUFFER_LEN, "%d", 
						MIN_RECOMMENDED_HDLC_MTU);
			SetDlgItemText(hdlg, EDIT_MTU, Globals_New_Cfg.aft_logic_ch_cfg[selected_ind].mtu);
		}
	}

	snprintf(local_tmp_buff, TMP_BUFFER_LEN, "aft_logic_channel_%d_mtu", selected_ind);
	SetRegistryStringValue(hkey, local_tmp_buff,
		(char*)	Globals_New_Cfg.aft_logic_ch_cfg[selected_ind].mtu);

	///////////////////////////////////////////////////////////////////
	//get operational mode from the page
	j = SendDlgItemMessage(hdlg, COMBO_OPERATIONAL_MODE, CB_GETCURSEL, 0, 0);
	strcpy(	Globals_New_Cfg.aft_logic_ch_cfg[selected_ind].operational_mode,
			s514_operational_mode_options[j]);

	snprintf(local_tmp_buff, TMP_BUFFER_LEN, "aft_logic_channel_%d_operational_mode", selected_ind);
	SetRegistryStringValue(hkey, local_tmp_buff,
		(char*)Globals_New_Cfg.aft_logic_ch_cfg[selected_ind].operational_mode);

	Debug(DBG_S514_PAGE, "*********  Finished writing S514 Interface Configuration  ********\n");
	///////////////////////////////////////////////////////////////////

	return TRUE;
}
///////////////////////////////////////////////////////////////////////////////

void S514_Init(HKEY	hkey, HWND hdlg, GLOBALS* cfg)
{
	int i;
	static char local_tmp_buff[TMP_BUFFER_LEN];

	///////////////////////////////////////////////////////////////////////
	//set Interface name on the page
	snprintf(local_tmp_buff, TMP_BUFFER_LEN, "%s_IF%d",
		Globals_New_Cfg.DeviceName, selected_ind);
	SetDlgItemText(hdlg, IF_NAME, local_tmp_buff);

	///////////////////////////////////////////////////////////////////////
	//set the MTU value
	SetDlgItemText(hdlg, EDIT_MTU, Globals_New_Cfg.aft_logic_ch_cfg[selected_ind].mtu);

	///////////////////////////////////////////////////////////////////////
	//add strings to Operational Mode combobox
	SendDlgItemMessage(hdlg, COMBO_OPERATIONAL_MODE, CB_RESETCONTENT, 0, 0);
	for(i = 0; i < NUM_OF_OPER_MODE_OPTIONS; i++){
		SendDlgItemMessage(hdlg, COMBO_OPERATIONAL_MODE, CB_ADDSTRING, 0,
			(LPARAM)s514_operational_mode_options[i]);
	}

	//make the current one selected
	i = SendDlgItemMessage(	hdlg, COMBO_OPERATIONAL_MODE, CB_FINDSTRING, -1,
							(LPARAM)Globals_New_Cfg.aft_logic_ch_cfg[selected_ind].operational_mode);
		
	SendDlgItemMessage(hdlg, COMBO_OPERATIONAL_MODE, CB_SETCURSEL, (i != CB_ERR ? i : 0), 0);

}

//////////////////////////////////////////////////////////////////////////////

